@@ -35,6 +35,10 @@ static CORBA_boolean
 impl_galeon_automation_quit (PortableServer_Servant _servant,
                              const CORBA_boolean disableServer,
                              CORBA_Environment * ev);
+static CORBA_boolean 
+impl_galeon_automation_load_session (PortableServer_Servant _servant,
+				     const CORBA_char * filename,
+				     CORBA_Environment * ev);
 static void
 galeon_automation_class_init (GaleonAutomationClass *klass);
 static void
@@ -173,6 +177,20 @@ impl_galeon_automation_quit (PortableServer_Servant _servant,
 	return retval;
 }
 
+static CORBA_boolean 
+impl_galeon_automation_load_session (PortableServer_Servant _servant,
+				     const CORBA_char * filename,
+				     CORBA_Environment * ev)
+{
+	CORBA_boolean retval = TRUE;
+	Session *session;
+
+	session = galeon_shell_get_session (galeon_shell);
+	session_load (session, filename);
+
+	return retval;
+}
+
 static void
 galeon_automation_class_init (GaleonAutomationClass *klass)
 {
@@ -187,6 +205,7 @@ galeon_automation_class_init (GaleonAutomationClass *klass)
         epv->loadurl = impl_galeon_automation_loadurl;
 	epv->addBookmark = impl_galeon_automation_add_bookmark;
 	epv->quit = impl_galeon_automation_quit;
+	epv->loadSession = impl_galeon_automation_load_session;
 }
 
 static void
