@@ -23,6 +23,8 @@
 #include "galeon-window.h"
 #include "galeon-sidebar.h"
 #include "gul-state.h"
+#include "gul-gobject-misc.h"
+#include "gul-notebook.h"
 #include "statusbar.h"
 #include "toolbar.h"
 #include "ppview-toolbar.h"
@@ -37,9 +39,12 @@
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
 #include "galeon-embed-utils.h"
+#include "galeon-embed-prefs.h"
 #include "bookmarks-editor-dockable.h"
 #include "window-recent-history-menu.h"
+#include "galeon-embed-prefs.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <bonobo/bonobo-window.h>
 #include <bonobo/bonobo-i18n.h>
@@ -66,6 +71,9 @@
 #define VIEW_TOOLBAR_PATH "/commands/View Toolbar"
 #define VIEW_BOOKMARKSBAR_PATH "/commands/View BookmarksBar"
 #define VIEW_FULLSCREEN_PATH "/commands/View Fullscreen"
+#define ALLOW_POPUPS_PATH "/commands/Allow Popups"
+#define ALLOW_JAVA_PATH "/commands/Allow Java"
+#define ALLOW_JAVASCRIPT_PATH "/commands/Allow JavaScript"
 
 #define ID_VIEW_SIDEBAR "View Sidebar"
 #define ID_VIEW_MENUBAR "View Menubar"
@@ -73,6 +81,21 @@
 #define ID_VIEW_TOOLBAR "View Toolbar"
 #define ID_VIEW_BOOKMARKSBAR "View BookmarksBar"
 #define ID_VIEW_FULLSCREEN "View Fullscreen"
+#define ID_ALLOW_POPUPS "Allow Popups"
+#define ID_ALLOW_JAVA "Allow Java"
+#define ID_ALLOW_JAVASCRIPT "Allow JavaScript"
+
+typedef enum {
+	IMAGES,
+	COOKIES
+}MenuRadioGroup;
+
+#define VIEW_IMAGES_ALWAYS	0
+#define VIEW_IMAGES_CURRENT	1
+#define VIEW_IMAGES_NEVER	2
+#define ACCEPT_COOKIES_ALWAYS	0
+#define ACCEPT_COOKIES_CURRENT	1
+#define ACCEPT_COOKIES_NEVER	2
 
 static BonoboUIVerb galeon_verbs [] = {
         BONOBO_UI_VERB ("EditFind", (BonoboUIVerbFn)window_cmd_edit_find),
@@ -86,6 +109,7 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("GoHome", (BonoboUIVerbFn)window_cmd_go_home),
 	BONOBO_UI_VERB ("GoMyportal", (BonoboUIVerbFn)window_cmd_go_myportal),
 	BONOBO_UI_VERB ("GoLocation", (BonoboUIVerbFn)window_cmd_go_location),
+	BONOBO_UI_VERB ("FileNew", (BonoboUIVerbFn)window_cmd_new),
 	BONOBO_UI_VERB ("FileNewWindow", (BonoboUIVerbFn)window_cmd_new_window),
 	BONOBO_UI_VERB ("FileNewTab", (BonoboUIVerbFn)window_cmd_new_tab),
 	BONOBO_UI_VERB ("FileOpen", (BonoboUIVerbFn)window_cmd_file_open),
@@ -106,6 +130,10 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("Zoom Out", (BonoboUIVerbFn)window_cmd_view_zoom_out),
 	BONOBO_UI_VERB ("Zoom Normal", (BonoboUIVerbFn)window_cmd_view_zoom_normal),
 	BONOBO_UI_VERB ("ViewPageSource", (BonoboUIVerbFn)window_cmd_view_page_source),
+//	BONOBO_UI_VERB ("BlockCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
+//	BONOBO_UI_VERB ("AllowCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("ShowJavaConsole", (BonoboUIVerbFn)window_cmd_show_java_console),
+	BONOBO_UI_VERB ("ShowJSConsole", (BonoboUIVerbFn)window_cmd_show_js_console),
 	BONOBO_UI_VERB ("BookmarksAddDefault", (BonoboUIVerbFn)window_cmd_bookmarks_add_default),
 	BONOBO_UI_VERB ("BookmarksEdit", (BonoboUIVerbFn)window_cmd_bookmarks_edit),
 	BONOBO_UI_VERB ("ToolsHistory", (BonoboUIVerbFn)window_cmd_tools_history),
@@ -114,6 +142,7 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("TabsPrevious", (BonoboUIVerbFn)window_cmd_tabs_previous),
 	BONOBO_UI_VERB ("TabsMoveLeft", (BonoboUIVerbFn)window_cmd_tabs_move_left),
 	BONOBO_UI_VERB ("TabsMoveRight", (BonoboUIVerbFn)window_cmd_tabs_move_right),
+	BONOBO_UI_VERB ("TabsDetach", (BonoboUIVerbFn)window_cmd_tabs_detach),
 	BONOBO_UI_VERB ("HelpContents", (BonoboUIVerbFn)window_cmd_help_manual),
 	BONOBO_UI_VERB ("About", (BonoboUIVerbFn)window_cmd_help_about),
 
@@ -140,6 +169,7 @@ struct GaleonWindowPrivate
 	Toolbar *toolbar;
 	Statusbar *statusbar;
 	GbBonoboUIMenu *bookmarks_menu;
+	GbBonoboUIMenu *bookmarks_context_menu;
 	GbBonoboUIToolbar *bookmarks_toolbar;
 	GtkNotebook *notebook;
 	GtkWidget *hpaned;
@@ -175,11 +205,15 @@ galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
 				       GaleonWindow *window);
 static void
 galeon_window_bookmark_activated_cb (GObject *sender,
-				     GbBookmark *b, const gchar *url, 
-				     GbBookmarkActivatedFlags f, GaleonWindow *w);
+				     GbBookmarkEventActivated *ev,
+				     GaleonWindow *w);
+
+static void             
+galeon_window_tab_detached_cb    (GulNotebook *notebook, gint page,
+				  gint x, gint y, GaleonWindow *window);
+
 void /* yes, this is not static */
-galeon_window_bookmark_activate (GaleonWindow *w, GbBookmark *b, const gchar *url, 
-				 GbBookmarkActivatedFlags f);
+galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev);
 
 static char *
 galeon_window_gb_location_source_get_location (GbLocationSource *src);
@@ -191,59 +225,37 @@ galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *w
 						const char *url, const char *title, 
 						GaleonWindow *w);
 
+/* static class variables */
+static GdkColor _galeon_window_loading_tab_color;
+static GdkColor _galeon_window_new_tab_color;
+
 static GObjectClass *parent_class = NULL;
 
-GType 
-galeon_window_get_type (void)
-{
-        static GType galeon_window_type = 0;
-
-        if (galeon_window_type == 0)
-        {
-                static const GTypeInfo our_info =
-			{
-				sizeof (GaleonWindowClass),
-				NULL, /* base_init */
-				NULL, /* base_finalize */
-				(GClassInitFunc) galeon_window_class_init,
-				NULL,
-				NULL, /* class_data */
-				sizeof (GaleonWindow),
-				0, /* n_preallocs */
-				(GInstanceInitFunc) galeon_window_init
-			};
-		
-		static const GInterfaceInfo gb_location_source_info =
-			{
-				(GInterfaceInitFunc) galeon_window_gb_location_source_init,
-				NULL,
-				NULL
-			};
-
-                galeon_window_type = g_type_register_static (BONOBO_TYPE_WINDOW,
-							     "GaleonWindow",
-							     &our_info, 0);
-		
-		g_type_add_interface_static (galeon_window_type,
-					     GB_TYPE_LOCATION_SOURCE,
-					     &gb_location_source_info);
-
-        }
-
-        return galeon_window_type;
-}
+MAKE_GET_TYPE_IFACE (galeon_window, "GaleonWindow", GaleonWindow,
+		     galeon_window_class_init, galeon_window_init, BONOBO_TYPE_WINDOW,
+		     galeon_window_gb_location_source_init, GB_TYPE_LOCATION_SOURCE);
 
 static void
 galeon_window_class_init (GaleonWindowClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+	char *tmp;
 	
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = galeon_window_finalize;
 	
 	widget_class->show = galeon_window_show;
+
+	/* initialize colors */
+	tmp = eel_gconf_get_string (CONF_TABS_TABBED_LOADING_COLOR);
+	gdk_color_parse (tmp, &_galeon_window_loading_tab_color);
+	g_free (tmp);
+
+	tmp = eel_gconf_get_string (CONF_TABS_TABBED_NEW_COLOR);
+	gdk_color_parse (tmp, &_galeon_window_new_tab_color);
+	g_free (tmp);
 }
 
 static void
@@ -374,6 +386,13 @@ setup_bonobo_window (GaleonWindow *window,
 			  window);
 }
 
+static void
+galeon_window_popup_before_show_cb (GaleonEmbedPopup *popup, GaleonWindow *window)
+{
+	/* we need this, because bonoboui sucks */
+	gb_bonobo_ui_menu_rebuild (window->priv->bookmarks_context_menu);
+}
+
 static GaleonEmbedPopupBW *
 setup_popup_factory (GaleonWindow *window, 
 		     BonoboUIComponent *ui_component)
@@ -386,6 +405,9 @@ setup_popup_factory (GaleonWindow *window,
                                 		     galeon_popup_verbs, 
 						     popup);
 
+	g_signal_connect (popup, "before-show", 
+			  G_CALLBACK (galeon_window_popup_before_show_cb), window);
+
 	return popup;
 }
 
@@ -393,16 +415,27 @@ static GtkNotebook *
 setup_notebook (GaleonWindow *window)
 {
 	GtkNotebook *notebook;
+	GtkPolicyType policy;
+
+	policy = eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW)
+		 ? GTK_POLICY_ALWAYS
+		 : GTK_POLICY_AUTOMATIC;
 	
 	notebook = GTK_NOTEBOOK (gul_notebook_new ());
 	gtk_notebook_set_scrollable (notebook, TRUE);
 	gtk_notebook_set_show_border (notebook, FALSE);
-	gtk_notebook_set_show_tabs (notebook, FALSE);
+	gul_notebook_set_policy (GUL_NOTEBOOK (notebook), policy);
+	gtk_notebook_popup_enable (notebook);
 
-	g_signal_connect_after (GTK_OBJECT (notebook), "switch_page",
-				GTK_SIGNAL_FUNC (
+	g_signal_connect_after (G_OBJECT (notebook), "switch_page",
+				G_CALLBACK (
 				galeon_window_notebook_switch_page_cb),
 				window);
+
+	g_signal_connect (G_OBJECT (notebook), "tab_detached", 
+			  G_CALLBACK (galeon_window_tab_detached_cb), 
+			  window);
+
 	gtk_widget_show (GTK_WIDGET (notebook));
 
 	return notebook;
@@ -415,7 +448,23 @@ setup_bookmarks_menu (GaleonWindow *window, GbBookmarkSet *bs)
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
 	
 	menu = gb_bonobo_ui_menu_new (bs->root, ui_component, 
-				      "/menu/Bookmarks/BookmarksPlaceholder");
+				      "/menu/Bookmarks/BookmarksPlaceholder", FALSE);
+	gb_bonobo_ui_menu_set_location_source (menu, GB_LOCATION_SOURCE (window));
+
+	g_signal_connect (menu, "bookmark-activated", 
+			  G_CALLBACK (galeon_window_bookmark_activated_cb), window);
+
+	return menu;
+}
+
+static GbBonoboUIMenu *
+setup_bookmarks_context_menu (GaleonWindow *window, GbBookmarkSet *bs)
+{
+	GbBonoboUIMenu *menu;
+	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
+	
+	menu = gb_bonobo_ui_menu_new (bs->root, ui_component, 
+				      "/popups/GaleonEmbedDocumentPopup/BookmarksInContext", TRUE);
 	gb_bonobo_ui_menu_set_location_source (menu, GB_LOCATION_SOURCE (window));
 
 	g_signal_connect (menu, "bookmark-activated", 
@@ -505,42 +554,234 @@ view_fullscreen_changed_cb (BonoboUIComponent *component,
 }
 
 static void
-update_layout_toggles (GaleonWindow *window)
+update_layout_toggles(GaleonWindow *window)
 {
-	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
+	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT(window->ui_component);
 	EmbedChromeMask mask = window->priv->chrome_mask;
 	
 	window->priv->ignore_layout_toggles = TRUE;
 
-	gul_bonobo_set_toggle_state (ui_component, VIEW_SIDEBAR_PATH, 
-				     mask & EMBED_CHROME_SIDEBARON);
-	gul_bonobo_set_toggle_state (ui_component, VIEW_TOOLBAR_PATH, 
-				     mask & EMBED_CHROME_TOOLBARON);
-	gul_bonobo_set_toggle_state (ui_component, VIEW_STATUSBAR_PATH, 
-				     mask & EMBED_CHROME_STATUSBARON);
-	gul_bonobo_set_toggle_state (ui_component, VIEW_FULLSCREEN_PATH, 
-				     mask & EMBED_CHROME_OPENASFULLSCREEN);
+	gul_bonobo_set_toggle_state(ui_component, VIEW_SIDEBAR_PATH, 
+				    mask & EMBED_CHROME_SIDEBARON);
+	gul_bonobo_set_toggle_state(ui_component, VIEW_TOOLBAR_PATH, 
+				    mask & EMBED_CHROME_TOOLBARON);
+	gul_bonobo_set_toggle_state(ui_component, VIEW_STATUSBAR_PATH, 
+				    mask & EMBED_CHROME_STATUSBARON);
+	gul_bonobo_set_toggle_state(ui_component, VIEW_FULLSCREEN_PATH, 
+				    mask & EMBED_CHROME_OPENASFULLSCREEN);
 
 	window->priv->ignore_layout_toggles = FALSE;
 }
 
 static void
-setup_layout_menus (GaleonWindow *window)
+setup_layout_menus(GaleonWindow *window)
+{
+	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT(window->ui_component);
+	
+	bonobo_ui_component_add_listener(ui_component, ID_VIEW_SIDEBAR, 
+					 (BonoboUIListenerFn)view_sidebar_changed_cb, 
+					 window);
+	bonobo_ui_component_add_listener(ui_component, ID_VIEW_TOOLBAR, 
+					 (BonoboUIListenerFn)view_toolbar_changed_cb, 
+					 window);
+	bonobo_ui_component_add_listener(ui_component, ID_VIEW_STATUSBAR, 
+					 (BonoboUIListenerFn)view_statusbar_changed_cb, 
+					 window);
+	bonobo_ui_component_add_listener(ui_component, ID_VIEW_FULLSCREEN, 
+					 (BonoboUIListenerFn)view_fullscreen_changed_cb, 
+					 window);
+}
+
+static void
+radio_menu_group_set_from_value(BonoboUIComponent *ui_component,
+				MenuRadioGroup group,
+				GConfValue *value)
+{
+	const char *path;
+
+	if (group == IMAGES)
+	{
+		switch (gconf_value_get_int(value))
+		{
+		  case VIEW_IMAGES_ALWAYS:
+		  default:
+			path = "/commands/ViewImagesAlways";
+			break;
+		  case VIEW_IMAGES_CURRENT:
+			path = "/commands/ViewImagesCurrent";
+			break;
+		  case VIEW_IMAGES_NEVER:
+			path = "/commands/ViewImagesNever";
+			break;
+		}
+	}
+	else //if (group == COOKIES)
+	{
+		switch (gconf_value_get_int(value))
+		{
+		  case ACCEPT_COOKIES_ALWAYS:
+		  default:
+			path = "/commands/AcceptCookiesAlways";
+			break;
+		  case ACCEPT_COOKIES_CURRENT:
+			path = "/commands/AcceptCookiesCurrent";
+			break;
+		  case ACCEPT_COOKIES_NEVER:
+			path = "/commands/AcceptCookiesNever";
+			break;
+		}
+	}
+
+	gul_bonobo_set_toggle_state(ui_component, path, TRUE);
+}
+
+static void
+allow_setting_bonoboui_changed_cb(BonoboUIComponent *component,
+				  const char *path,
+				  Bonobo_UIComponent_EventType type,
+				  const char *state,
+				  const char *prefs_string)
+{
+	if (state == NULL || *state == '\0') return;
+
+	eel_gconf_set_boolean(prefs_string, strcmp(state, "1") == 0);
+}
+
+static void
+allow_setting_gconf_changed_cb(GConfClient *client,
+			       guint cnxn_id,
+			       GConfEntry *entry,
+			       BonoboUIComponent *ui_component)
+{
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
+}
+
+static void
+accept_cookies_bonoboui_changed_cb(BonoboUIComponent *component,
+				   const char *id,
+				   Bonobo_UIComponent_EventType type,
+				   const char *state,
+				   gpointer user_data)
+{
+	if (state == NULL || *state == '\0') return;
+
+	/* we only care when an item being set */
+	if (!strcmp(state, "0")) return;
+
+	eel_gconf_set_integer(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
+			      GPOINTER_TO_INT(user_data));
+}
+
+static void
+accept_cookies_gconf_changed_cb(GConfClient *client,
+				guint cnxn_id,
+				GConfEntry *entry,
+				BonoboUIComponent *ui_component)
+{
+	radio_menu_group_set_from_value(ui_component, COOKIES, entry->value);
+}
+
+static void
+setup_web_menu (GaleonWindow *window)
 {
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
-	
-	bonobo_ui_component_add_listener (ui_component, ID_VIEW_SIDEBAR, 
-					  (BonoboUIListenerFn)view_sidebar_changed_cb, 
-					  window);
-	bonobo_ui_component_add_listener (ui_component, ID_VIEW_TOOLBAR, 
-					  (BonoboUIListenerFn)view_toolbar_changed_cb, 
-					  window);
-	bonobo_ui_component_add_listener (ui_component, ID_VIEW_STATUSBAR, 
-					  (BonoboUIListenerFn)view_statusbar_changed_cb, 
-					  window);
-	bonobo_ui_component_add_listener (ui_component, ID_VIEW_FULLSCREEN, 
-					  (BonoboUIListenerFn)view_fullscreen_changed_cb, 
-					  window);
+
+	radio_menu_group_set_from_value(ui_component, COOKIES,
+					eel_gconf_get_value(CONF_PERSISTENT_COOKIES_BEHAVIOR));
+
+	bonobo_ui_component_add_listener(ui_component, "AcceptCookiesAlways",
+					 accept_cookies_bonoboui_changed_cb,
+					 GINT_TO_POINTER(ACCEPT_COOKIES_ALWAYS));
+	bonobo_ui_component_add_listener(ui_component, "AcceptCookiesCurrent",
+					 accept_cookies_bonoboui_changed_cb,
+					 GINT_TO_POINTER(ACCEPT_COOKIES_CURRENT));
+	bonobo_ui_component_add_listener(ui_component, "AcceptCookiesNever",
+					 accept_cookies_bonoboui_changed_cb,
+					 GINT_TO_POINTER(ACCEPT_COOKIES_NEVER));
+
+	eel_gconf_notification_add(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
+				   (GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
+				   ui_component);
+
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
+				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
+
+	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_POPUPS,
+					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
+					 CONF_FILTERING_ALLOW_POPUPS);
+	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_JAVASCRIPT,
+					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
+					 CONF_FILTERING_JAVASCRIPT_ENABLED);
+	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_JAVA,
+					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
+					 CONF_FILTERING_JAVA_ENABLED);
+
+	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				   ui_component);
+	eel_gconf_notification_add(CONF_FILTERING_JAVASCRIPT_ENABLED,
+				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				   ui_component);
+	eel_gconf_notification_add(CONF_FILTERING_JAVA_ENABLED,
+				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				   ui_component);
+}
+
+static void
+view_images_bonoboui_changed_cb(BonoboUIComponent *component,
+			        const char *id,
+			        Bonobo_UIComponent_EventType type,
+			        const char *state,
+			        gpointer user_data)
+{
+	if (state == NULL || *state == '\0') return;
+
+	/* we only care when an item being set */
+	if (!strcmp(state, "0")) return;
+
+	eel_gconf_set_integer(CONF_FILTERING_IMAGE_LOADING_TYPE, 
+			      GPOINTER_TO_INT(user_data));
+}
+
+static void
+view_images_gconf_changed_cb(GConfClient *client,
+		             guint cnxn_id,
+			     GConfEntry *entry,
+			     BonoboUIComponent *ui_component)
+{
+	radio_menu_group_set_from_value(ui_component, IMAGES, entry->value);
+}
+
+static void
+setup_images_menu(GaleonWindow *window)
+{
+	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT(window->ui_component);
+
+	radio_menu_group_set_from_value(ui_component, IMAGES,
+					eel_gconf_get_value(CONF_FILTERING_IMAGE_LOADING_TYPE));
+
+	bonobo_ui_component_add_listener(ui_component, "ViewImagesAlways",
+					 view_images_bonoboui_changed_cb,
+					 GINT_TO_POINTER(VIEW_IMAGES_ALWAYS));
+	bonobo_ui_component_add_listener(ui_component, "ViewImagesCurrent",
+					 view_images_bonoboui_changed_cb,
+					 GINT_TO_POINTER(VIEW_IMAGES_CURRENT));
+	bonobo_ui_component_add_listener(ui_component, "ViewImagesNever",
+					 view_images_bonoboui_changed_cb,
+					 GINT_TO_POINTER(VIEW_IMAGES_NEVER));
+
+	eel_gconf_notification_add(CONF_FILTERING_IMAGE_LOADING_TYPE, 
+				   (GConfClientNotifyFunc)view_images_gconf_changed_cb, 
+				   ui_component);
 }
 
 static void
@@ -662,6 +903,8 @@ galeon_window_init (GaleonWindow *window)
                                 		     galeon_verbs, 
 						     window);
 	setup_layout_menus (window);
+	setup_images_menu (window);
+	setup_web_menu(window);
 	
 	/* Setup the embed popups factory */
 	window->priv->embed_popup = setup_popup_factory (window, 
@@ -685,6 +928,7 @@ galeon_window_init (GaleonWindow *window)
 	/* Setup bookmarks ui elements */
 	bs = galeon_shell_get_bookmark_set (galeon_shell);
 	window->priv->bookmarks_menu = setup_bookmarks_menu (window, bs); 
+	window->priv->bookmarks_context_menu = setup_bookmarks_context_menu (window, bs); 
 	window->priv->bookmarks_toolbar = setup_bookmarks_toolbar (window, bs); 	
 	
 	/* Setup window contents */
@@ -762,6 +1006,7 @@ galeon_window_finalize (GObject *object)
 	}
 
 	g_object_unref (G_OBJECT (window->priv->bookmarks_menu));
+	g_object_unref (G_OBJECT (window->priv->bookmarks_context_menu));
 	g_object_unref (G_OBJECT (window->priv->bookmarks_toolbar));
 	g_object_unref (G_OBJECT (window->priv->toolbar));
 	g_object_unref (G_OBJECT (window->priv->statusbar));
@@ -981,12 +1226,6 @@ galeon_window_set_chrome (GaleonWindow *window,
 	save_window_chrome (window);
 }
 
-GtkWidget *
-galeon_window_get_notebook (GaleonWindow *window)
-{
-	return GTK_WIDGET (window->priv->notebook);
-}
-
 void 
 galeon_window_add_tab (GaleonWindow *window, 
 		       GaleonTab *tab,
@@ -1003,7 +1242,7 @@ galeon_window_add_tab (GaleonWindow *window,
 
 	gul_notebook_insert_page (GUL_NOTEBOOK (window->priv->notebook), 
 				  widget,
-				  -1,
+				  GUL_NOTEBOOK_INSERT_GROUPED,
 				  jump_to);
 }
 
@@ -1023,26 +1262,31 @@ galeon_window_jump_to_tab (GaleonWindow *window,
 }
 
 static GaleonTab *
-real_get_active_tab (GaleonWindow *window, int page_num)
+get_tab_from_page_num (GtkNotebook *notebook, gint page_num) 
 {
-	GaleonTab *tab;	
+	GaleonTab *tab;
 	GtkWidget *embed_widget;
 
+	if (page_num < 0) return NULL;
+	
+	embed_widget = gtk_notebook_get_nth_page (notebook, page_num);
+	
+	g_return_val_if_fail (GTK_IS_WIDGET (embed_widget), NULL);
+	tab = g_object_get_data (G_OBJECT (embed_widget), "GaleonTab");
+	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);
+	
+	return tab;
+}
+
+static GaleonTab *
+real_get_active_tab (GaleonWindow *window, int page_num)
+{
 	if (page_num == -1)
 	{
 		page_num = gtk_notebook_get_current_page (window->priv->notebook);
 	}
 
-	if (page_num < 0) return NULL;
-	
-	embed_widget = gtk_notebook_get_nth_page 
-		(window->priv->notebook, page_num);
-		
-	g_return_val_if_fail (GTK_IS_WIDGET (embed_widget), NULL);
-	tab = g_object_get_data (G_OBJECT (embed_widget), "GaleonTab");
-	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);
-
-	return tab;
+	return get_tab_from_page_num (window->priv->notebook, page_num);
 }
 
 void 
@@ -1189,9 +1433,11 @@ update_security (GaleonWindow *window)
 	embed = galeon_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	level = STATE_IS_INSECURE;
-	description = NULL;
-	galeon_embed_get_security_level (embed, &level, &description);
+	if (galeon_embed_get_security_level (embed, &level, &description) != G_OK)
+	{
+		level = STATE_IS_UNKNOWN;
+		description = NULL;
+	}
 
 	secure = FALSE;
 	switch (level)
@@ -1263,7 +1509,6 @@ update_nav_control (GaleonWindow *window)
 	forward = galeon_embed_can_go_forward (embed);
 	up = galeon_embed_can_go_up (embed);
 	stop = galeon_tab_get_load_status (tab) & TAB_LOAD_STARTED;
-	galeon_embed_zoom_get (embed, &zoom);
 
 	toolbar_button_set_sensitive (window->priv->toolbar, 
 				      TOOLBAR_BACK_BUTTON,
@@ -1277,8 +1522,10 @@ update_nav_control (GaleonWindow *window)
 	toolbar_button_set_sensitive (window->priv->toolbar, 
 				      TOOLBAR_STOP_BUTTON,
 				      stop);
-	toolbar_set_zoom (window->priv->toolbar, zoom);
-	
+	if (galeon_embed_zoom_get (embed, &zoom) == G_OK)
+	{
+		toolbar_set_zoom (window->priv->toolbar, zoom);
+	}
 
 	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
 			          GO_BACK_CMD_PATH, !back);
@@ -1351,10 +1598,10 @@ update_find_control (GaleonWindow *window)
 static void
 update_window_visibility (GaleonWindow *window)
 {
-	GList *l;
+	GList *tabs, *l;
 	
-	l = galeon_window_get_tabs (window);
-	for (; l != NULL; l = l->next)
+	tabs = galeon_window_get_tabs (window);
+	for (l = tabs; l != NULL; l = l->next)
 	{
 		GaleonTab *tab = GALEON_TAB(l->data);
 		g_return_if_fail (IS_GALEON_TAB(tab));
@@ -1362,10 +1609,11 @@ update_window_visibility (GaleonWindow *window)
 		if (galeon_tab_get_visibility (tab))
 		{
 			gtk_widget_show (GTK_WIDGET(window));
+			g_list_free (tabs);
 			return;
 		}
 	}
-	g_list_free (l);
+	g_list_free (tabs);
 
 	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window)))
 	{
@@ -1376,10 +1624,10 @@ update_window_visibility (GaleonWindow *window)
 static void
 update_spinner_control (GaleonWindow *window)
 {
-	GList *l;
+	GList *tabs, *l;
 
-	l = galeon_window_get_tabs (window);
-	for (; l != NULL; l = l->next)
+	tabs = galeon_window_get_tabs (window);
+	for (l = tabs; l != NULL; l = l->next)
 	{
 		GaleonTab *tab = GALEON_TAB(l->data);
 		g_return_if_fail (IS_GALEON_TAB(tab));
@@ -1387,10 +1635,11 @@ update_spinner_control (GaleonWindow *window)
 		if (galeon_tab_get_load_status (tab) & TAB_LOAD_STARTED)
 		{
 			toolbar_spinner_start (window->priv->toolbar);
+			g_list_free (tabs);
 			return;
 		}
 	}
-	g_list_free (l);
+	g_list_free (tabs);
 	
 	toolbar_spinner_stop (window->priv->toolbar);
 }
@@ -1456,6 +1705,7 @@ galeon_window_update_all_controls (GaleonWindow *window)
 		update_favicon_control (window);
 		update_status_message (window);
 		update_progress (window);
+		update_security (window);
 		update_find_control (window);
 		update_spinner_control (window);
 	}
@@ -1582,9 +1832,8 @@ galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
 {
 	GaleonTab *tab, *old_tab;
 
-	if (window->priv->closing) return;
-	
 	g_return_if_fail (IS_GALEON_WINDOW (window));
+	if (window->priv->closing) return;
 
 	/* get the new tab */
 	tab = real_get_active_tab (window, page_num);
@@ -1613,71 +1862,93 @@ galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
  * used in galeon-window-bookmark-activated-proxy.c (and only there: it's not public)
  */
 void
-galeon_window_bookmark_activate (GaleonWindow *w, GbBookmark *b, const gchar *url, 
-				 GbBookmarkActivatedFlags f)
+galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev)
 {
-	GaleonTab *tab;
-	tab = galeon_window_get_active_tab (w);
+	GaleonTab *tab = galeon_window_get_active_tab (w);
+	GbBookmark *b = ev->bookmark;
+	const char *url = ev->url;
+	GbBookmarkActivatedFlags f = ev->flags;
+	GaleonNewTabFlags flags = 0;
+
+	if (GB_IS_SITE (b) && f == GB_BAF_DEFAULT)
+	{
+		galeon_window_load_url (w, url);
+		return;
+	}
+
+	if (GB_IS_FOLDER (b) && f == GB_BAF_DEFAULT)
+	{
+		f = GB_BAF_NEW_TAB_OR_WINDOW;
+	}
+
+	if (f == GB_BAF_NEW_WINDOW)
+	{
+		flags = GALEON_NEW_TAB_IN_NEW_WINDOW;
+	}
+	else if (f == GB_BAF_NEW_TAB)
+	{
+		flags = GALEON_NEW_TAB_IN_EXISTING_WINDOW;
+	}
+	else if (f == GB_BAF_NEW_TAB_OR_WINDOW)
+	{
+		GdkModifierType mod;
+		gdk_event_get_state (ev->event, &mod);
+		flags = galeon_shell_modifier_flags (mod);
+	}
+	else
+	{
+		g_assert_not_reached ();
+	}
 
 	if (GB_IS_SITE (b))
 	{
-		if (f == GB_BAF_DEFAULT)
+		galeon_shell_new_tab (galeon_shell, w, tab, url, flags);
+	}
+	else if (GB_IS_FOLDER (b) && flags & GALEON_NEW_TAB_IN_NEW_WINDOW)
+	{
+		GbBookmark *bi;
+		for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
 		{
-			galeon_window_load_url (w, url);
-		}
-		else if (f == GB_BAF_NEW_WINDOW)
-		{
-			galeon_shell_new_tab (galeon_shell, NULL, tab, url, 
-					      GALEON_NEW_TAB_IN_NEW_WINDOW);
-		} 
-		else if (f == GB_BAF_NEW_TAB)
-		{
-			galeon_shell_new_tab (galeon_shell, w, tab, url, 
-					      GALEON_NEW_TAB_IN_EXISTING_WINDOW);
-		}
-		else if (f == GB_BAF_NEW_TAB_OR_WINDOW)
-		{
-			galeon_shell_new_tab (galeon_shell, w, tab, url, 0);
+			if (GB_IS_SITE (bi))
+			{
+				galeon_shell_new_tab (galeon_shell, w, tab, 
+						      GB_SITE (bi)->url, flags);
+			}
 		}
 	}
 	else if (GB_IS_FOLDER (b))
 	{
-		if (f == GB_BAF_NEW_TAB || f == GB_BAF_DEFAULT)
+		GbBookmark *bi;
+		GaleonTab *jump_to_tab;
+		gboolean jump_to;
+		
+		jump_to = flags & GALEON_NEW_TAB_JUMP;
+		if (flags & GALEON_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
+
+		/* Open all bookmarks in order in the background.  Jumping
+		 * to them automatically is a major performance hit with
+		 * >3 bookmarks in a folder.  Once all bookmarks are opened,
+		 * we can efficiently jump to the first one if 'jump to
+		 * new tabs automatically' is enabled.
+		 */
+		flags |= GALEON_NEW_TAB_DONT_JUMP_TO;
+
+		jump_to_tab = NULL;
+		for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
 		{
-			GbBookmark *bi;
-			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
+			if (GB_IS_SITE (bi))
 			{
-				if (GB_IS_SITE (bi))
-				{
-					tab = galeon_shell_new_tab (galeon_shell, w, tab,
-								    GB_SITE (bi)->url, 
-								    GALEON_NEW_TAB_IN_EXISTING_WINDOW);
-				}
+				tab = galeon_shell_new_tab (galeon_shell, w, tab,
+							    GB_SITE (bi)->url, 
+							    flags);
+
+				if (jump_to_tab == NULL) jump_to_tab = tab;
 			}
 		}
-		else if (f == GB_BAF_NEW_WINDOW)
+
+		if (jump_to && jump_to_tab != NULL)
 		{
-			GbBookmark *bi;
-			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
-			{
-				if (GB_IS_SITE (bi))
-				{
-					galeon_shell_new_tab (galeon_shell, NULL, tab, GB_SITE (bi)->url, 
-							      GALEON_NEW_TAB_IN_NEW_WINDOW);
-				}
-			}
-		}
-		else if (f == GB_BAF_NEW_TAB_OR_WINDOW)
-		{
-			GbBookmark *bi;
-			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
-			{
-				if (GB_IS_SITE (bi))
-				{
-					galeon_shell_new_tab (galeon_shell, w, tab, GB_SITE (bi)->url, 
-							      0);
-				}
-			}
+			galeon_window_jump_to_tab (w, jump_to_tab);
 		}
 	}
 	else
@@ -1688,10 +1959,10 @@ galeon_window_bookmark_activate (GaleonWindow *w, GbBookmark *b, const gchar *ur
 
 static void
 galeon_window_bookmark_activated_cb (GObject *sender,
-				     GbBookmark *b, const gchar *url, 
-				     GbBookmarkActivatedFlags f, GaleonWindow *w)
+				     GbBookmarkEventActivated *ev,
+				     GaleonWindow *w)
 {
-	galeon_window_bookmark_activate (w, b, url, f);
+	galeon_window_bookmark_activate (w, ev);
 }
 
 static void
@@ -1823,3 +2094,71 @@ galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *w
 	galeon_window_load_url (w, url);
 }
 
+void
+galeon_window_tab_detached_cb (GulNotebook *notebook, gint page, 
+			        gint x, gint y, GaleonWindow *window)
+{
+	GaleonWindow *new_win;
+	GaleonTab *tab;
+
+	tab = get_tab_from_page_num (GTK_NOTEBOOK (notebook), page);
+	new_win = galeon_window_new ();
+
+	galeon_window_reparent_tab (new_win, window, tab);
+
+	gtk_widget_show (GTK_WIDGET (new_win));
+}
+
+void
+galeon_window_reparent_tab (GaleonWindow *window, GaleonWindow *source,
+		            GaleonTab *tab)
+{
+	GtkWidget *src_page;
+
+	src_page = GTK_WIDGET (galeon_tab_get_embed (tab));
+	gul_notebook_move_page (GUL_NOTEBOOK (source->priv->notebook),
+				GUL_NOTEBOOK (window->priv->notebook),
+				src_page, 0);
+	galeon_tab_set_window (tab, window);
+}
+
+void
+galeon_window_set_tab_status (GaleonWindow	*window,
+		              GaleonTab		*tab,
+			      TabLoadStatus	 status)
+{
+	GdkColor *color;
+
+	switch (status)
+	{
+	case TAB_LOAD_STARTED:
+		color = &_galeon_window_loading_tab_color;
+        	break;
+	case TAB_LOAD_COMPLETED:
+		color = &_galeon_window_new_tab_color;
+		break;
+	case TAB_LOAD_NONE:
+	default:
+		color = NULL;
+		break;
+	}
+
+	gul_notebook_set_page_color (GUL_NOTEBOOK (window->priv->notebook),
+			             GTK_WIDGET (galeon_tab_get_embed (tab)),
+				     color);
+}
+
+void
+galeon_window_set_tab_title (GaleonWindow	*window,
+		             GaleonTab		*tab,
+			     const char 	*title)
+{
+	gul_notebook_set_page_title (GUL_NOTEBOOK (window->priv->notebook),
+				     GTK_WIDGET (galeon_tab_get_embed (tab)),
+				     title);
+
+	if (galeon_window_get_active_tab (window) == tab)
+	{
+		galeon_window_update_control (window, TitleControl);
+	}
+}
