@@ -34,6 +34,14 @@
 #include "gul-toolbar-bonobo-view.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
+#include "gul-toolbar-item-factory.h"
+#include "gul-tbi-zoom.h"
+#include "galeon-navigation-button.h"
+#include "galeon-reload-button.h"
+#include "galeon-go-button.h"
+#include "galeon-home-button.h"
+#include "galeon-new-button.h"
+#include "galeon-stop-button.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
@@ -45,15 +53,12 @@
 #include <gtk/gtkmenu.h>
 
 #define DEFAULT_TOOLBAR_SETUP \
-	"back=std_toolitem(item=back);" \
-	"back_history=navigation_history(direction=back);" \
-	"up=std_toolitem(item=up);" \
-	"up_history=navigation_history(direction=up);" \
-	"forward=std_toolitem(item=forward);" \
-	"forward_history=navigation_history(direction=forward);" \
-	"stop=std_toolitem(item=stop);" \
-	"reload=std_toolitem(item=reload);" \
-	"home=std_toolitem(item=home);" \
+	"back_menu=navigation_button(direction=back,arrow=TRUE);" \
+	"up_menu=navigation_button(direction=up,arrow=TRUE);" \
+	"forward_menu=navigation_button(direction=forward,arrow=TRUE);" \
+	"stop=stop_button;" \
+	"reload=reload_button;" \
+	"home=home_button;" \
 	"favicon=favicon;" \
 	"location=location;" \
 	"zoom=zoom;" \
@@ -61,6 +66,8 @@
 
 #define ZOOM_DELAY 50
 
+#define GALEON_HOMEPAGE "http://galeon.sourceforge.net/"
+
 static void toolbar_class_init (ToolbarClass *klass);
 static void toolbar_init (Toolbar *t);
 static void toolbar_finalize (GObject *object);
@@ -102,12 +109,13 @@ struct ToolbarPrivate
 
 	GtkWidget *spinner;
 	gboolean visibility;
-	/* This field is unused... what is it?
-	  GdkPixbufAnimation *animation;
-	*/
-	GtkWidget *back_button;
-	GtkWidget *forward_button;
-	GtkWidget *up_button;
+
+	GSList *navigation_buttons;
+	GaleonReloadButton *reload_button;
+	GaleonGoButton *go_button;
+	GaleonHomeButton *home_button;
+	GaleonStopButton *stop_button;
+	GaleonNewButton *new_button;
 	GtkWidget *location_entry;
 	GtkTooltips *tooltips;
 	GtkWidget *favicon;
@@ -165,6 +173,13 @@ toolbar_class_init (ToolbarClass *klass)
                                                               "Parent window",
                                                               GALEON_WINDOW_TYPE,
                                                               G_PARAM_READWRITE));
+
+	gul_toolbar_item_register_type ("navigation_button", (GulTbItemConstructor) galeon_navigation_button_new);
+	gul_toolbar_item_register_type ("reload_button", (GulTbItemConstructor) galeon_reload_button_new);
+	gul_toolbar_item_register_type ("go_button", (GulTbItemConstructor) galeon_go_button_new);
+	gul_toolbar_item_register_type ("home_button", (GulTbItemConstructor) galeon_home_button_new);
+	gul_toolbar_item_register_type ("stop_button", (GulTbItemConstructor) galeon_stop_button_new);
+	gul_toolbar_item_register_type ("new_button", (GulTbItemConstructor) galeon_new_button_new);
 }
 
 static void
@@ -177,10 +192,10 @@ toolbar_set_property (GObject *object,
 
         switch (prop_id)
         {
-                case PROP_GALEON_WINDOW:
-                        toolbar_set_window (t, g_value_get_object (value));
-                        break;
-        }
+	case PROP_GALEON_WINDOW: 
+		toolbar_set_window (t, g_value_get_object (value));
+		break;
+	}
 }
 
 static void
@@ -193,259 +208,12 @@ toolbar_get_property (GObject *object,
 
         switch (prop_id)
         {
-                case PROP_GALEON_WINDOW:
-                        g_value_set_object (value, t->priv->window);
-                        break;
+	case PROP_GALEON_WINDOW:
+		g_value_set_object (value, t->priv->window);
+		break;
         }
 }
 
-static GtkWidget *
-new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, 
-                         GtkWidget *menu, const GdkPixbuf *ico)
-{
-        gchar *text = gul_string_new_num_accel (num, origtext, lettersok);
-        GtkWidget *item = gtk_image_menu_item_new ();
-        GtkWidget *hb = gtk_hbox_new (FALSE, 0);
-        GtkWidget *label = gtk_label_new_with_mnemonic (text);
-
-        gtk_box_pack_start (GTK_BOX (hb), label, FALSE, FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (item), hb);
-
-        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                       gtk_image_new_from_pixbuf ((GdkPixbuf *) ico));
-
-        gtk_widget_show_all (item);
-
-        return item;
-}
-
-static void
-activate_back_or_forward_menu_item_cb (GtkWidget *menu, GaleonWindow *window)
-{
-	GaleonEmbed *embed;
-	int go_nth;
-	
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	go_nth = (int)g_object_get_data (G_OBJECT(menu), "go_nth");
-
-	galeon_embed_shistory_go_nth (embed, go_nth);
-}
-
-static void
-activate_up_menu_item_cb (GtkWidget *menu, GaleonWindow *window)
-{
-	GaleonEmbed *embed;
-	int go_nth;
-	GSList *l;
-	gchar *url;
-	
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	go_nth = (int)g_object_get_data (G_OBJECT(menu), "go_nth");
-
-	galeon_embed_get_go_up_list (embed, &l);
-
-	url = g_slist_nth_data (l, go_nth);
-	if (url)
-	{
-		galeon_embed_load_url (embed, url);
-	}
-
-	g_slist_foreach (l, (GFunc) g_free, NULL);
-	g_slist_free (l);
-}
-
-static gboolean
-back_or_forward_button_pressed_callback (GtkWidget *widget, 
-					 GdkEventButton *event,
-					 gpointer *user_data)
-{
-	Toolbar *t;
-	GtkWidget *menu;
-	int pos, count;
-	GaleonEmbed *embed;
-	int start, end, accell_count = 0;
-
-	g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
-
-	t = TOOLBAR (user_data);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-
-	embed = galeon_window_get_active_embed (t->priv->window);
-	g_return_val_if_fail (embed != NULL, FALSE);
-	
-	galeon_embed_shistory_get_pos (embed, &pos);
-	galeon_embed_shistory_count (embed, &count);
-
-	if (count == 0) return FALSE;
-	
-	if (widget == t->priv->back_button)
-	{
-		start = pos - 1;
-		end = -1;
-	}
-	else
-	{
-		start = pos + 1;
-		end = count;
-	}
-
-	menu = gtk_menu_new ();
-
-	while (start != end)
-	{
-		char *title, *url, *tmp;
-		GtkWidget *item;
-		galeon_embed_shistory_get_nth (embed, start, FALSE,
-					       &url, &title);
-		tmp = gul_string_escape_uline_accel (title);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, 
-						menu, NULL);
-        	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_object_set_data (G_OBJECT(item), "go_nth", GINT_TO_POINTER (start));
-		g_signal_connect (item, "activate",
-                                  G_CALLBACK (activate_back_or_forward_menu_item_cb),
-                                  t->priv->window);
-		gtk_widget_show_all (item);
-
-		g_free (url);
-		g_free (title);
-		g_free (tmp);
-
-		accell_count++;
-		if (start < end) start++;
-		else start--;
-	}
-
-
-	gnome_popup_menu_do_popup_modal (menu,
-					 gul_gui_menu_position_under_widget, widget, event, widget, widget);
-	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	
-	return TRUE;
-}
-
-static gboolean 
-up_button_pressed_callback (GtkWidget *widget, 
-			    GdkEventButton *event,
-			    gpointer *user_data)
-{
-	Toolbar *t;
-	GtkWidget *menu;
-	GaleonEmbed *embed;
-	int accell_count = 0;
-	GSList *l;
-	GSList *li;
-
-	g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
-
-	t = TOOLBAR (user_data);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-
-	embed = galeon_window_get_active_embed (t->priv->window);
-	g_return_val_if_fail (embed != NULL, FALSE);
-	
-	galeon_embed_get_go_up_list (embed, &l);
-
-	if (l == NULL) return FALSE;
-	
-	menu = gtk_menu_new ();
-	
-	for (li = l; li; li = li->next)
-	{
-		char *url = li->data;
-		char *tmp;
-		GtkWidget *item;
-		tmp = gul_string_escape_uline_accel (url);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, 
-						menu, NULL);
-        	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_object_set_data (G_OBJECT(item), "go_nth", GINT_TO_POINTER (accell_count));
-		g_signal_connect (item, "activate",
-                                  G_CALLBACK (activate_up_menu_item_cb),
-                                  t->priv->window);
-		gtk_widget_show_all (item);
-
-		accell_count++;
-		g_free (tmp);
-	}
-	
-	g_slist_foreach (l, (GFunc) g_free, NULL);
-	g_slist_free (l);
-
-	gnome_popup_menu_do_popup_modal (menu,
-					 gul_gui_menu_position_under_widget, widget, event, widget, widget);
-	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	
-	return TRUE;
-}
-
-static gboolean
-back_or_forward_key_pressed_callback (GtkWidget *widget,
-				      GdkEventKey *event,
-				      gpointer *user_data)
-{
-	if (event->keyval == GDK_space ||
-	    event->keyval == GDK_KP_Space ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter)
-	{
-		back_or_forward_button_pressed_callback (widget, NULL, user_data);
-	}
-
-	return FALSE;
-}
-
-static gboolean
-up_key_pressed_callback (GtkWidget *widget,
-			 GdkEventKey *event,
-			 gpointer *user_data)
-{
-	if (event->keyval == GDK_space ||
-	    event->keyval == GDK_KP_Space ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter)
-	{
-		up_button_pressed_callback (widget, NULL, user_data);
-	}
-
-	return FALSE;
-}
-
-static void
-toolbar_setup_navigation_button (Toolbar *t, GtkWidget *w, const char *tooltip)
-{
-	g_signal_connect_object (w, "key_press_event",
-				 G_CALLBACK (back_or_forward_key_pressed_callback),
-				 t, 0);
-	g_signal_connect_object (w, "button_press_event",
-				 G_CALLBACK (back_or_forward_button_pressed_callback),
-				 t, 0);
-	
-	gtk_tooltips_set_tip (t->priv->tooltips, w, tooltip, NULL);
-}
-
-static void 
-toolbar_setup_up_button (Toolbar *t, GtkWidget *w, const char *tooltip)
-{
-	g_signal_connect_object (w, "key_press_event",
-				 G_CALLBACK (up_key_pressed_callback),
-				 t, 0);
-	g_signal_connect_object (w, "button_press_event",
-				 G_CALLBACK (up_button_pressed_callback),
-				 t, 0);
-	
-	gtk_tooltips_set_tip (t->priv->tooltips, w, tooltip, NULL);
-}
-
-
 static void
 toolbar_location_url_activate_cb (GaleonLocationEntry *entry, const gchar *url, GaleonWindow *window)
 {
@@ -519,6 +287,7 @@ static void
 toolbar_zoom_spinbutton_value_changed_cb (GtkSpinButton *sb, Toolbar *t)
 {
 	ToolbarPrivate *p = t->priv;
+	
 	if (p->zoom_timeout_id != 0)
 	{
 		g_source_remove (p->zoom_timeout_id);
@@ -534,7 +303,6 @@ toolbar_setup_zoom_spinbutton (Toolbar *t, GtkWidget *w)
 {
 	g_signal_connect (w, "value_changed", 
 			  G_CALLBACK (toolbar_zoom_spinbutton_value_changed_cb), t);
-	gtk_tooltips_set_tip (t->priv->tooltips, w, _("Zoom"), NULL);
 }
 
 static void
@@ -550,10 +318,21 @@ toolbar_setup_location_entry (Toolbar *t, GtkWidget *w)
 	galeon_location_entry_set_autocompletion (e, ac);
 
 	g_signal_connect (e, "url-activated",
-			  GTK_SIGNAL_FUNC(toolbar_location_url_activate_cb),
+			  G_CALLBACK (toolbar_location_url_activate_cb),
 			  t->priv->window);
 }
 	
+static gboolean
+toolbar_spinner_button_release_event_cb (GtkWidget *w, GdkEventButton *ev, Toolbar *t)
+{
+	if (ev->button >= 0 && ev->button <= 3)
+	{
+		ToolbarPrivate *p = t->priv;
+		galeon_window_load_url (p->window, GALEON_HOMEPAGE);
+	}
+	return FALSE;
+}
+
 static void
 toolbar_setup_spinner (Toolbar *t, GtkWidget *w)
 {
@@ -571,8 +350,43 @@ toolbar_setup_spinner (Toolbar *t, GtkWidget *w)
 	/* don't care about the box anymore */
 	g_object_unref (p->spinner);
 	p->spinner = g_object_ref (spinner);
+
+	g_signal_connect (p->spinner, "button-release-event",
+			  G_CALLBACK (toolbar_spinner_button_release_event_cb), t);
 }
 
+static void
+toolbar_setup_style (Toolbar *t)
+{
+	static const char *styles[] = { "both_horiz", "both", "text", "icon" };
+	const char *style;
+	GConfValue *value;
+
+	g_return_if_fail (t->priv->ui_component != NULL);
+
+	style = "";
+	value = eel_gconf_get_value (CONF_TOOLBAR_STYLE);
+	if (value != NULL)
+	{
+		guint i = gconf_value_get_int (value);
+		if (i < G_N_ELEMENTS(styles))
+		{
+			style = styles[i];
+		}
+		gconf_value_free (value);
+	}
+
+	gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
+}
+
+static void
+toolbar_style_changed_cb (GConfClient *client,
+		          guint cnxn_id,
+			  GConfEntry *entry,
+			  gpointer t)
+{
+	toolbar_setup_style (TOOLBAR (t));
+}
 
 static void
 toolbar_set_window (Toolbar *t, GaleonWindow *window)
@@ -584,6 +398,10 @@ toolbar_set_window (Toolbar *t, GaleonWindow *window)
 
 	gul_tb_bonobo_view_set_path (t->priv->bview, t->priv->ui_component, "/Toolbar");
 
+	eel_gconf_notification_add (CONF_TOOLBAR_STYLE,
+			            toolbar_style_changed_cb, t);
+
+	toolbar_setup_style (t);
 	toolbar_get_widgets (t);
 }
 
@@ -593,7 +411,10 @@ toolbar_get_widgets (Toolbar *t)
 	ToolbarPrivate *p;
 	GulToolbar *gt;
 	GulTbItem *it;
-	
+	GSList *li;
+	const gchar *nav_buttons_ids[] = {"back", "back_menu", "up", "up_menu", "forward", "forward_menu" };
+	guint i;
+
 	DEBUG_MSG (("in toolbar_get_widgets\n"));
 
 	g_return_if_fail (IS_TOOLBAR (t));
@@ -603,24 +424,43 @@ toolbar_get_widgets (Toolbar *t)
 
 	/* release all the widgets */
 
-	if (p->back_button)
+	for (li = p->navigation_buttons; li; li = li->next)
 	{
-		g_object_unref (p->back_button);
-		p->back_button = NULL;
+		g_object_unref (li->data);
 	}
+	g_slist_free (p->navigation_buttons);
+	p->navigation_buttons = NULL;
 
-	if (p->forward_button)
+	if (p->reload_button)
 	{
-		g_object_unref (p->forward_button);
-		p->forward_button = NULL;
+		g_object_unref (p->reload_button);
 	}
+	p->reload_button = NULL;
 
-	if (p->up_button)
+	if (p->go_button)
 	{
-		g_object_unref (p->up_button);
-		p->up_button = NULL;
+		g_object_unref (p->go_button);
 	}
-	
+	p->go_button = NULL;
+
+	if (p->home_button)
+	{
+		g_object_unref (p->home_button);
+	}
+	p->home_button = NULL;
+
+	if (p->stop_button)
+	{
+		g_object_unref (p->stop_button);
+	}
+	p->stop_button = NULL;
+
+	if (p->new_button)
+	{
+		g_object_unref (p->new_button);
+	}
+	p->new_button = NULL;
+
 	if (p->favicon_ebox)
 	{
 		g_object_unref (p->favicon_ebox);
@@ -653,34 +493,121 @@ toolbar_get_widgets (Toolbar *t)
 	
 	gt = GUL_TOOLBAR (t);
 
-	it = gul_toolbar_get_item_by_id (gt, "back_history");
-	if (it)
+	for (i = 0; i < G_N_ELEMENTS (nav_buttons_ids); ++i)
 	{
-		p->back_button = gul_tb_item_get_widget (it);
-		g_object_ref (p->back_button);
-		toolbar_setup_navigation_button (t, p->back_button, _("Go back a number of pages"));
-		
-		DEBUG_MSG (("    got a back_history button\n"));
-	}
-
-	it = gul_toolbar_get_item_by_id (gt, "forward_history");
-	if (it)
-	{
-		p->forward_button = gul_tb_item_get_widget (it);
-		g_object_ref (p->forward_button);
-		toolbar_setup_navigation_button (t, p->forward_button, _("Go forward a number of pages"));
-
-		DEBUG_MSG (("    got a forward_history button\n"));
+		it = gul_toolbar_get_item_by_id (gt, nav_buttons_ids[i]);
+		if (it)
+		{
+			if (GALEON_IS_NAVIGATION_BUTTON (it))
+			{	
+				DEBUG_MSG (("    got a navigation button\n"));
+				p->navigation_buttons = g_slist_prepend (p->navigation_buttons, g_object_ref (it));
+				if (p->window)
+				{
+					galeon_tbi_set_window (GALEON_TBI (it), p->window);
+				}
+			}
+			else
+			{
+				g_warning ("An unexpected button has been found in your toolbar. "
+					   "Maybe your setup is too old.");
+			}
+		}
 	}
 	
-	it = gul_toolbar_get_item_by_id (gt, "up_history");
+	it = gul_toolbar_get_item_by_id (gt, "reload");
 	if (it)
 	{
-		p->up_button = gul_tb_item_get_widget (it);
-		g_object_ref (p->up_button);
-		toolbar_setup_up_button (t, p->up_button, _("Go up a number of levels"));
-		
-		DEBUG_MSG (("    got a up_history button\n"));
+		if (GALEON_IS_RELOAD_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a reload button\n"));
+			p->reload_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
+	it = gul_toolbar_get_item_by_id (gt, "go");
+	if (it)
+	{
+		if (GALEON_IS_GO_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a go button\n"));
+			p->go_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
+	it = gul_toolbar_get_item_by_id (gt, "home");
+	if (it)
+	{
+		if (GALEON_IS_HOME_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a home button\n"));
+			p->home_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
+	it = gul_toolbar_get_item_by_id (gt, "stop");
+	if (it)
+	{
+		if (GALEON_IS_STOP_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a stop button\n"));
+			p->stop_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
+	it = gul_toolbar_get_item_by_id (gt, "new");
+	if (it)
+	{
+		if (GALEON_IS_NEW_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a new button\n"));
+			p->new_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
 	}
 
 	it = gul_toolbar_get_item_by_id (gt, "location");
@@ -716,7 +643,7 @@ toolbar_get_widgets (Toolbar *t)
 	it = gul_toolbar_get_item_by_id (gt, "zoom");
 	if (it)
 	{
-		p->zoom_spinbutton = gul_tb_item_get_widget (it);
+		p->zoom_spinbutton = gul_tbi_zoom_get_spinner (GUL_TBI_ZOOM (it));
 		g_object_ref (p->zoom_spinbutton);
 		toolbar_setup_zoom_spinbutton (t, p->zoom_spinbutton);
 
@@ -735,6 +662,7 @@ toolbar_init (Toolbar *t)
 	t->priv->window = NULL;
 	t->priv->ui_component = NULL;
 	t->priv->visibility = TRUE;
+	t->priv->navigation_buttons = NULL;
 	t->priv->tooltips = gtk_tooltips_new ();
 	g_object_ref (t->priv->tooltips);
 	gtk_object_sink (GTK_OBJECT (t->priv->tooltips));
@@ -771,6 +699,7 @@ toolbar_finalize (GObject *object)
 {
 	Toolbar *t;
 	ToolbarPrivate *p;
+	GSList *li;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (IS_TOOLBAR (object));
@@ -781,9 +710,6 @@ toolbar_finalize (GObject *object)
         g_return_if_fail (p != NULL);
 
 	if (p->location_entry) g_object_unref (p->location_entry);
-	if (p->back_button) g_object_unref (p->back_button);
-	if (p->forward_button) g_object_unref (p->forward_button);
-	if (p->up_button) g_object_unref (p->up_button);
 	if (p->favicon_ebox) g_object_unref (p->favicon_ebox);
 	if (p->favicon) g_object_unref (p->favicon);
 	if (p->spinner) g_object_unref (p->spinner);
@@ -795,9 +721,42 @@ toolbar_finalize (GObject *object)
 	}
 
 	g_object_unref (t->priv->bview);
+	
+	for (li = t->priv->navigation_buttons; li; li = li->next)
+	{
+		g_object_unref (li->data);
+	}
+	g_slist_free (t->priv->navigation_buttons);
+
+	if (t->priv->reload_button)
+	{
+		g_object_unref (p->reload_button);
+	}
+
+	if (t->priv->go_button)
+	{
+		g_object_unref (p->go_button);
+	}
+
+	if (t->priv->home_button)
+	{
+		g_object_unref (p->home_button);
+	}
+
+	if (t->priv->stop_button)
+	{
+		g_object_unref (p->stop_button);
+	}
+
+	if (t->priv->new_button)
+	{
+		g_object_unref (p->new_button);
+	}
 
         g_free (t->priv);
 
+	DEBUG_MSG (("Toolbar finalized\n"));
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -855,46 +814,43 @@ toolbar_spinner_stop (Toolbar *t)
 	}
 }
 
+static void
+toolbar_navigation_button_set_sensitive (Toolbar *t, GaleonNavigationDirection d, gboolean sensitivity)
+{
+	GSList *li;
+	ToolbarPrivate *p = t->priv;
+
+	for (li = p->navigation_buttons; li; li = li->next)
+	{
+		GaleonNavigationButton *b = GALEON_NAVIGATION_BUTTON (li->data);
+		if (galeon_navigation_button_get_direction (b) == d)
+		{
+			galeon_navigation_button_set_sensitive (b, sensitivity);
+		}
+	}
+}
+
 void 
 toolbar_button_set_sensitive (Toolbar *t,
 			      ToolbarButtonID id,
 			      gboolean sensitivity)
 {
+	ToolbarPrivate *p = t->priv;
 	switch (id)
 	{
 	case TOOLBAR_BACK_BUTTON:
-		gul_bonobo_set_sensitive (t->priv->ui_component,
-					  "/commands/GoBack",
-					  sensitivity);
-		if (t->priv->back_button)
-		{
-			gtk_widget_set_sensitive (t->priv->back_button,
-						  sensitivity);
-		}
+		toolbar_navigation_button_set_sensitive (t, GALEON_NAVIGATION_DIRECTION_BACK, sensitivity);
 		break;
 	case TOOLBAR_FORWARD_BUTTON:
-		gul_bonobo_set_sensitive (t->priv->ui_component,
-					  "/commands/GoForward",
-					  sensitivity);
-		if (t->priv->forward_button)
-		{
-			gtk_widget_set_sensitive (t->priv->forward_button,
-						  sensitivity);
-		}
-		break;
-	case TOOLBAR_STOP_BUTTON:
-		gul_bonobo_set_sensitive (t->priv->ui_component,
-					  "/commands/GoStop",
-					  sensitivity);
+		toolbar_navigation_button_set_sensitive (t, GALEON_NAVIGATION_DIRECTION_FORWARD, sensitivity);
 		break;
 	case TOOLBAR_UP_BUTTON:
-		gul_bonobo_set_sensitive (t->priv->ui_component,
-					  "/commands/GoUp",
-					  sensitivity);
-		if (t->priv->up_button)
+		toolbar_navigation_button_set_sensitive (t, GALEON_NAVIGATION_DIRECTION_UP, sensitivity);
+		break;
+	case TOOLBAR_STOP_BUTTON:
+		if (p->stop_button)
 		{
-			gtk_widget_set_sensitive (t->priv->up_button,
-						  sensitivity);
+			galeon_stop_button_set_sensitive (p->stop_button, sensitivity);
 		}
 		break;
 	}
