@@ -40,20 +40,22 @@
 #include <libgnomeui/gnome-about.h>
 #include <libgnome/gnome-help.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkeditable.h>
 
 #define AVAILABLE_TOOLBAR_ITEMS \
-	"back=std_toolitem(item=back);" \
-	"back_history=navigation_history(direction=back);" \
-	"up=std_toolitem(item=up);" \
-	"up_history=navigation_history(direction=up);" \
-	"forward=std_toolitem(item=forward);" \
-	"forward_history=navigation_history(direction=forward);" \
-	"stop=std_toolitem(item=stop);" \
-	"reload=std_toolitem(item=reload);" \
-	"home=std_toolitem(item=home);" \
+	"new=new_button;" \
+	"back=navigation_button(direction=back,arrow=FALSE);" \
+	"back_menu=navigation_button(direction=back,arrow=TRUE);" \
+	"forward=navigation_button(direction=forward,arrow=FALSE);" \
+	"forward_menu=navigation_button(direction=forward,arrow=TRUE);" \
+	"up=navigation_button(direction=up,arrow=FALSE);" \
+	"up_menu=navigation_button(direction=up,arrow=TRUE);" \
+	"stop=stop_button;" \
+	"reload=reload_button;" \
+	"home=home_button;" \
 	"favicon=favicon;" \
 	"location=location;" \
-	"go=std_toolitem(item=go);" \
+	"go=go_button;" \
 	"zoom=zoom;" \
 	"spinner=spinner;" \
 	"separator;"
@@ -265,6 +267,20 @@ window_cmd_go_reload (BonoboUIComponent *uic,
 	galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
 }
 
+void
+window_cmd_new (BonoboUIComponent *uic, 
+		GaleonWindow *window, 
+		const char* verbname)
+{
+	GaleonTab *tab;
+
+	tab = galeon_window_get_active_tab (window);
+	
+	galeon_shell_new_tab (galeon_shell, window, tab, NULL, 
+			      GALEON_NEW_TAB_HOMEPAGE |
+			      GALEON_NEW_TAB_JUMP);
+}
+
 void
 window_cmd_new_window (BonoboUIComponent *uic, 
 		       GaleonWindow *window, 
@@ -310,8 +326,6 @@ window_cmd_bookmarks_edit (BonoboUIComponent *uic,
 	e = gb_editor_new_for_set (set);
 	gtk_widget_show (gb_editor_get_main_widget (e));
 
-	gb_editor_set_location_source (e, GB_LOCATION_SOURCE (window));
-	
 	wbap = galeon_window_bookmark_activated_proxy_new ();
 	galeon_window_bookmark_activated_proxy_set_window (wbap, window);
 	galeon_window_bookmark_activated_proxy_connect (wbap, G_OBJECT (e));
@@ -457,12 +471,20 @@ window_cmd_edit_cut (BonoboUIComponent *uic,
 		     GaleonWindow *window, 
 		     const char* verbname)
 {
-	GaleonEmbed *embed;
+	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-	
-	galeon_embed_selection_cut (embed);
+	if (GTK_IS_EDITABLE (widget))
+	{
+		gtk_editable_cut_clipboard (GTK_EDITABLE (widget));
+	}
+	else
+	{
+		GaleonEmbed *embed;
+		embed = galeon_window_get_active_embed (window);
+		g_return_if_fail (embed != NULL);
+		
+		galeon_embed_selection_cut (embed);
+	}
 }
 
 void
@@ -470,12 +492,21 @@ window_cmd_edit_copy (BonoboUIComponent *uic,
 		      GaleonWindow *window, 
 		      const char* verbname)
 {
-	GaleonEmbed *embed;
+	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-	
-	galeon_embed_selection_copy (embed);
+	if (GTK_IS_EDITABLE (widget))
+	{
+		gtk_editable_copy_clipboard (GTK_EDITABLE (widget));
+	}
+	else
+	{
+		GaleonEmbed *embed;
+
+		embed = galeon_window_get_active_embed (window);
+		g_return_if_fail (embed != NULL);
+		
+		galeon_embed_selection_copy (embed);
+	}
 }
 
 void
@@ -483,12 +514,21 @@ window_cmd_edit_paste (BonoboUIComponent *uic,
 		       GaleonWindow *window, 
 		       const char* verbname)
 {
-	GaleonEmbed *embed;
+	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-	
-	galeon_embed_paste (embed);
+	if (GTK_IS_EDITABLE (widget))
+	{
+		gtk_editable_paste_clipboard (GTK_EDITABLE (widget));
+	}
+	else
+	{
+		GaleonEmbed *embed;
+
+		embed = galeon_window_get_active_embed (window);
+		g_return_if_fail (embed != NULL);
+		
+		galeon_embed_paste (embed);
+	}
 }
 
 void 
@@ -496,12 +536,21 @@ window_cmd_edit_select_all (BonoboUIComponent *uic,
 		      	    GaleonWindow *window, 
 		      	    const char* verbname)
 {
-	GaleonEmbed *embed;
+	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-	
-	galeon_embed_select_all (embed);
+	if (GTK_IS_EDITABLE (widget))
+	{
+		gtk_editable_select_region (GTK_EDITABLE (widget), 0, -1);
+	}
+	else
+	{
+		GaleonEmbed *embed;
+
+		embed = galeon_window_get_active_embed (window);
+		g_return_if_fail (embed != NULL);
+		
+		galeon_embed_select_all (embed);
+	}
 }
 
 void
@@ -513,7 +562,7 @@ window_cmd_edit_find_next (BonoboUIComponent *uic,
 	
 	dialog = galeon_window_get_find_dialog (window);
 
-	find_dialog_go_next (FIND_DIALOG(dialog));
+	find_dialog_go_next (FIND_DIALOG(dialog), FALSE);
 
 	galeon_window_update_control (window, FindControl);
 }
@@ -527,11 +576,83 @@ window_cmd_edit_find_prev (BonoboUIComponent *uic,
 	
 	dialog = galeon_window_get_find_dialog (window);
 
-	find_dialog_go_prev (FIND_DIALOG(dialog));
+	find_dialog_go_prev (FIND_DIALOG(dialog), FALSE);
 
 	galeon_window_update_control (window, FindControl);
 }
 
+void 
+window_cmd_set_permission(BonoboUIComponent *uic, 
+		      	  GaleonWindow *window, 
+			  const char* verbname)
+{
+	gresult result;
+
+	gchar *url;
+	PermissionType type;
+	gboolean value;
+
+	GaleonEmbed *embed;
+	GaleonEmbedShell *embed_shell;
+
+	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
+	g_return_if_fail(embed_shell != NULL);
+
+	embed = galeon_window_get_active_embed(window);
+	result = galeon_embed_get_location(embed, TRUE, FALSE, &url);
+	if(result != G_OK) return;
+
+	if (!strcmp(verbname, "AllowCookieSite"))
+	{
+		type = COOKIES_PERMISSION;
+		value = TRUE;
+	}
+	else if (!strcmp(verbname, "BlockCookieSite"))
+	{
+		type = COOKIES_PERMISSION;
+		value = FALSE;
+	}
+	else if (!strcmp(verbname, "AllowImageSite"))
+	{
+		type = IMAGES_PERMISSION;
+		value = TRUE;
+	}
+	else //if (!strcmp(verbname, "BlockImageSite"))
+	{
+		type = IMAGES_PERMISSION;
+		value = FALSE;
+	}
+
+	galeon_embed_shell_set_permission(embed_shell, url, type, value);
+}
+
+void
+window_cmd_show_java_console(BonoboUIComponent *uic,
+			     GaleonWindow *window,
+			     const char* verbname)
+{
+	GaleonEmbedShell *embed_shell;
+	gresult result;
+
+	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
+	g_return_if_fail(embed_shell != NULL);
+
+	result = galeon_embed_shell_show_java_console(embed_shell);
+}
+
+void window_cmd_show_js_console(BonoboUIComponent *uic,
+				GaleonWindow *window,
+				const char* verbname)
+{
+	GaleonEmbedShell *embed_shell;
+	gresult result;
+
+	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
+	g_return_if_fail(embed_shell != NULL);
+
+	result = galeon_embed_shell_show_js_console(embed_shell);
+}
+
 void 
 window_cmd_view_zoom_in	(BonoboUIComponent *uic, 
 		      	 GaleonWindow *window, 
@@ -726,9 +847,7 @@ window_cmd_help_about (BonoboUIComponent *uic,
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
 		"Remco de Jong <rdj@rdj.cg.nu>",
-		/* if your encoding allows it, use aacute (U00E1)
-		 * for the 'a' of 'Fernandez' */
-		N_("Ricardo Fernandez Pascual <ric@users.sourceforge.net>"),
+		"Ricardo Fern\xc3\xa1ndez Pascual <ric@users.sourceforge.net>",
 		"Nate Case <nd@kracked.com>",
 		"Bob Phan <ex-freek@yahoo.com>",
 		"Matt Aubury <matt@ookypooky.com>",
@@ -739,11 +858,10 @@ window_cmd_help_about (BonoboUIComponent *uic,
 		"Matthew Mueller <donut@azstarnet.com>",
 		"Daniel Erat <dan_erat@pobox.com>",
 		"Jorn Baayen <jorn@nl.linux.org>",
-		" ", /* dont delete the space */
-		N_("Graphic arts: "),
-		/* if your encoding allows it, use eacute (U00E9)
-		 * for the two 'e's of Frederic */
-		N_("Toussaint Frederic <fred@linuxgraphic.org>"),
+		/* don't delete the space */
+		" ",
+		N_("Graphic arts:"),
+		"Toussaint Fr\xc3\xa9" "d\xc3\xa9ric <fred@linuxgraphic.org>",
 		NULL
 	};
 
@@ -772,11 +890,7 @@ window_cmd_help_about (BonoboUIComponent *uic,
 	
 	about = gnome_about_new(
 		       _("Galeon"), VERSION,
-		       /* Translators: Please change the (C) to a real
-			* copyright character if your character set allows it
-			* (Hint: iso-8859-1 is one of the character sets that
-			* has this symbol). */
-		       _("Copyright (C) 2000-2002 Marco Pesenti Gritti"),
+		       "Copyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
 		       _("A GNOME browser based on Mozilla"),
 		       (const char **)authors,
 		       (const char **)documenters,
@@ -900,6 +1014,26 @@ void window_cmd_tabs_move_right (BonoboUIComponent *uic,
 	}
 }
 
+void 
+window_cmd_tabs_detach  (BonoboUIComponent *uic, 
+			 GaleonWindow *window, 
+			 const char* verbname)
+{
+	GaleonWindow *new_win;
+	GaleonTab *tab;
+
+	if (g_list_length (galeon_window_get_tabs (window)) <= 1) {
+		return;
+	}
+
+	tab = galeon_window_get_active_tab (window);	
+	new_win = galeon_window_new ();
+
+	galeon_window_reparent_tab (new_win, window, tab);
+
+	gtk_widget_show (GTK_WIDGET (new_win));
+}
+
 void 
 window_cmd_help_manual (BonoboUIComponent *uic, 
 			char *filename, 
