@@ -393,7 +393,7 @@ static void
 prefs_connect_signals (GaleonDialog *dialog)
 {
 	int i;
-	GSList *list;
+	const GSList *list;
 	PropertyInfo *props = dialog->priv->props;
 	
 	for (i = 0; props[i].widget != NULL; i++)
@@ -874,11 +874,15 @@ impl_show (GaleonDialog *dialog)
 	
 	if (dialog->priv->parent)
 	{
-		/* make the dialog transient again, because it seems to get
-		 * forgotten after gtk_widget_hide
+		/* Make the dialog transient again, because it seems to get
+		 * forgotten after gtk_widget_hide.  Stray dialogs aren't of
+		 * much use (and we may also risk a crash) so let's make sure
+		 * our dialog gets destroyed along with its parent.
 		 */
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->priv->dialog),
 					      GTK_WINDOW (dialog->priv->parent));
+		gtk_window_set_destroy_with_parent 
+			(GTK_WINDOW(dialog->priv->dialog), TRUE);
 	}
 	gtk_window_present (GTK_WINDOW(dialog->priv->dialog));
 }
